<?php
// +----------------------------------------------------------------------
// | ThinkPHP [ WE CAN DO IT JUST THINK ]
// +----------------------------------------------------------------------
// | Copyright (c) 2006~2018 http://thinkphp.cn All rights reserved.
// +----------------------------------------------------------------------
// | Licensed ( http://www.apache.org/licenses/LICENSE-2.0 )
// +----------------------------------------------------------------------
// | Author: liu21st <liu21st@gmail.com>
// +----------------------------------------------------------------------

namespace think\route\dispatch;

use think\Container;
use think\exception\ClassNotFoundException;
use think\exception\HttpException;
use think\Loader;
use think\route\Dispatch;

class Module extends Dispatch
{
    public function run()
    {
        $result = $this->dispatch;
        // 判断是否为字符
        if (is_string($result)) {
            $result = explode('/', $result);
        }
        
        // $this->app   think\App
        // app.app_multi_module     是否支持多模块 默认为true
        if ($this->app->config('app.app_multi_module')) {
            // 多模块部署
            // strip_tags() 去除字符串中的 HTML 标签
            // strtolower() 所有字符转换为小写
            // app.default_module 默认模块名 默认为index
            $module    = strip_tags(strtolower($result[0] ?: $this->app->config('app.default_module')));
            // $this->app['route']    think\Route
            // 读取路由绑定
            $bind      = $this->app['route']->getBind();

            $available = false;
            // 检测是否有路由规则
            // 取消大小写敏感性,那么默认的"."代表除了换行符以外的任何字符将会变成任意字符，也就是包括换行符.
            if ($bind && preg_match('/^[a-z]/is', $bind)) {
                // 绑定模块
                list($bindModule) = explode('/', $bind);
                if (empty($result[0])) {
                    $module = $bindModule;
                }
                $available = true;
            } elseif (!in_array($module, $this->app->config('app.deny_module_list')) && is_dir($this->app->getAppPath() . $module)) {
                // app.deny_module_list ['common'] 禁止访问模块 ,检测$module 是否在禁止访问的模块内
                // is_dir() 函数检查指定的文件是否是目录    $this->app->getAppPath() D:\wamp\www\tp5\application/
                // 没有路由规则默认执行这个
                $available = true;
            } elseif ($this->app->config('app.empty_module')) {
                // app.empty_module 默认的空模块名
                $module    = $this->app->config('app.empty_module');
                $available = true;
            }

            // 模块初始化
            if ($module && $available) {
                // 初始化模块
                $this->app['request']->module($module);
                //对模块进行初始化
                $this->app->init($module);

                // $this->app->getAppPath()   D:\wamp\www\tp5\application/
                // 加载当前模块语言包
                $this->app['lang']->load($this->app->getAppPath() . $module . '/lang/' . $this->app['request']->langset() . '.php');

                // 模块请求缓存检查，默认不会执行缓存
                $this->app['request']->cache(
                    // app.request_cache 默认为false 是否开启请求缓存 true自动缓存 支持设置请求缓存规则
                    $this->app->config('app.request_cache'),
                    // app.request_cache_expire 默认为null 请求缓存有效期
                    $this->app->config('app.request_cache_expire'),
                    // app.request_cache_except 默认为[] 全局请求缓存排除规则
                    $this->app->config('app.request_cache_except')
                );
            } else {
                throw new HttpException(404, 'module not exists:' . $module);
            }
        } else {
            // 单一模块部署
            $module = '';
            $this->app['request']->module($module);
        }

        // 当前模块路径
        // $this->app->getAppPath()   D:\wamp\www\tp5\application/
        $this->app->setModulePath($this->app->getAppPath() . ($module ? $module . '/' : ''));

        // 是否自动转换控制器和操作名
        // app.url_convert 默认为true 是否自动转换URL中的控制器和操作名
        $convert = is_bool($this->convert) ? $this->convert : $this->app->config('app.url_convert');
        // 获取控制器名
        // strip_tags() 剥去字符串中的 HTML 标签
        $controller = strip_tags($result[1] ?: $this->app->config('app.default_controller'));
        // 将控制器名转换成小写
        $controller = $convert ? strtolower($controller) : $controller;

        // 获取操作名
        $actionName = strip_tags($result[2] ?: $this->app->config('app.default_action'));
        // 将操作名转换成小写
        $actionName = $convert ? strtolower($actionName) : $actionName;

        // 设置当前请求的控制器、操作
        // Loader::parseName($controller, 1) 将首字母转成大写
        $this->app['request']->controller(Loader::parseName($controller, 1))->action($actionName);

        // 监听module_init
        $this->app['hook']->listen('module_init');

        var_dump('------------------');
//        exit;
        // 实例化控制器
        try {
            $instance = $this->app->controller($controller,
                $this->app->config('app.url_controller_layer'),
                $this->app->config('app.controller_suffix'),
                $this->app->config('app.empty_controller')
            );
        } catch (ClassNotFoundException $e) {
            throw new HttpException(404, 'controller not exists:' . $e->getClass());
        }

        //进行钩子执行
        // 获取当前操作名
        $action = $actionName . $this->app->config('app.action_suffix');

        if (is_callable([$instance, $action])) {
            // 执行操作方法
            $call = [$instance, $action];
            // 自动获取请求变量
            $vars = $this->app->config('app.url_param_type')
            ? $this->app['request']->route()
            : $this->app['request']->param();
        } elseif (is_callable([$instance, '_empty'])) {
            // 空操作
            $call = [$instance, '_empty'];
            $vars = [$actionName];
        } else {
            // 操作不存在
            throw new HttpException(404, 'method not exists:' . get_class($instance) . '->' . $action . '()');
        }

        $this->app['hook']->listen('action_begin', $call);
       // Container::getInstance()  think\Container
        // invokeMethod() 调用反射执行类的方法 支持参数绑定
        return Container::getInstance()->invokeMethod($call, $vars);
    }
}
